/*
 * epgsearch.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <ctype.h>
#include <vdr/plugin.h>
#include <menu.h>
#include <vdr/status.h>
#include <vdr/epg.h>
#include "i18n.h"
#include "epgsearchsetup.h"
#include "epgsearchcfg.h"
#include "epgsearchext.h"

static const char *VERSION        = "0.0.1";
static const char *DESCRIPTION    = "search the EPG for repeatings and more";
static const char *MAINMENUENTRY  = "EPG search";

#define CHNUMWIDTH  (numdigits(Channels.MaxNumber()) + 1)

static int CompareEventTime(const void *p1, const void *p2)
{
  return (int)((*(cEvent **)p1)->StartTime() - (*(cEvent **)p2)->StartTime());
}

static int CompareEventChannel(const void *p1, const void *p2)
{
  return (int)( Channels.GetByChannelID((*(const cEvent **)p1)->ChannelID())->Number() - Channels.GetByChannelID((*(const cEvent **)p2)->ChannelID())->Number());
}

void ToLower(char* szText)
{
	if (!szText)
		return;
	for (int loop = 0; szText[loop] !=0; loop++)
        szText[loop] = tolower(szText[loop]);
}

cMenuEditTimer::cMenuEditTimer(cTimer *Timer, bool New)
:cOsdMenu(tr("Edit timer"), 12)
{
  firstday = NULL;
  timer = Timer;
  addIfConfirmed = New;
  if (timer) {
     data = *timer;
     if (New)
        data.SetFlags(tfActive);
     channel = data.Channel()->Number();
     Add(new cMenuEditBitItem( tr("Active"),       &data.flags, tfActive));
     Add(new cMenuEditChanItem(tr("Channel"),      &channel));
     Add(new cMenuEditDayItem( tr("Day"),          &data.day));
     Add(new cMenuEditTimeItem(tr("Start"),        &data.start));
     Add(new cMenuEditTimeItem(tr("Stop"),         &data.stop));
     Add(new cMenuEditBitItem( tr("VPS"),          &data.flags, tfVps));
     Add(new cMenuEditIntItem( tr("Priority"),     &data.priority, 0, MAXPRIORITY));
     Add(new cMenuEditIntItem( tr("Lifetime"),     &data.lifetime, 0, MAXLIFETIME));
     Add(new cMenuEditStrItem( tr("File"),          data.file, sizeof(data.file), tr(FileNameChars)));
     SetFirstDayItem();
     }
  Timers.IncBeingEdited();
}

cMenuEditTimer::~cMenuEditTimer()
{
  if (timer && addIfConfirmed)
     delete timer; // apparently it wasn't confirmed
  Timers.DecBeingEdited();
}

void cMenuEditTimer::SetFirstDayItem(void)
{
  if (!firstday && !data.IsSingleEvent()) {
     Add(firstday = new cMenuEditDateItem(tr("First day"), &data.firstday));
     Display();
     }
  else if (firstday && data.IsSingleEvent()) {
     Del(firstday->Index());
     firstday = NULL;
     data.firstday = 0;
     Display();
     }
}

eOSState cMenuEditTimer::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  if (state == osUnknown) {
     switch (Key) {
       case kOk:     {
                       cChannel *ch = Channels.GetByNumber(channel);
                       if (ch)
                          data.channel = ch;
                       else {
                          Skins.Message(mtError, tr("*** Invalid Channel ***"));
                          break;
                          }
                       if (!*data.file)
                          strcpy(data.file, data.Channel()->Name());
                       if (timer) {
                          if (memcmp(timer, &data, sizeof(data)) != 0) {
                             *timer = data;
                             if (timer->HasFlags(tfActive))
                                timer->ClrFlags(~tfAll); // allows external programs to mark active timers with values > 0xFFFF and recognize if the user has modified them
                             }
                          if (addIfConfirmed)
                             Timers.Add(timer);
                          timer->Matches();
                          Timers.Save();
                          isyslog("timer %d %s (%s)", timer->Index() + 1, addIfConfirmed ? "added" : "modified", timer->HasFlags(tfActive) ? "active" : "inactive");
                          addIfConfirmed = false;
                          }
                     }
                     return osBack;
       case kRed:
       case kGreen:
       case kYellow:
       case kBlue:   return osContinue;
       default: break;
       }
     }
  if (Key != kNone)
     SetFirstDayItem();
  return state;
}

// --- cMenuEventSearch ------------------------------------------------------------

class cMenuEventSearch : public cOsdMenu {
private:
  const cEvent *event;
public:
  cMenuEventSearch(const cEvent *Event, bool CanSwitch = false);
  virtual void Display(void);
  virtual eOSState ProcessKey(eKeys Key);
};

cMenuEventSearch::cMenuEventSearch(const cEvent *Event, bool CanSwitch)
:cOsdMenu(tr("Event"))
{
  event = Event;
  if (event) {
     cChannel *channel = Channels.GetByChannelID(event->ChannelID(), true);
     if (channel) {
        SetTitle(channel->Name());
        SetHelp(tr("Record"), NULL, NULL, CanSwitch ? tr("Switch") : NULL);
        }
     }
}

void cMenuEventSearch::Display(void)
{
  cOsdMenu::Display();
  DisplayMenu()->SetEvent(event);
  cStatus::MsgOsdTextItem(event->Description());
}

eOSState cMenuEventSearch::ProcessKey(eKeys Key)
{
  switch (Key) {
    case kUp|k_Repeat:
    case kUp:
    case kDown|k_Repeat:
    case kDown:
    case kLeft|k_Repeat:
    case kLeft:
    case kRight|k_Repeat:
    case kRight:
                  DisplayMenu()->Scroll(NORMALKEY(Key) == kUp || NORMALKEY(Key) == kLeft, NORMALKEY(Key) == kLeft || NORMALKEY(Key) == kRight);
                  cStatus::MsgOsdTextItem(NULL, NORMALKEY(Key) == kUp);
                  return osContinue;
    default: break;
    }

  eOSState state = cOsdMenu::ProcessKey(Key);

  if (state == osUnknown) {
     switch (Key) {
       case kGreen:
       case kYellow: return osContinue;
       case kOk:     return osBack;
       default: break;
       }
     }
  return state;
}

// --- cMenuWhatsOnSearchItem ------------------------------------------------------

class cMenuWhatsOnSearchItem : public cOsdItem {
public:
  const cEvent *event;
  const cChannel *channel;
  cMenuWhatsOnSearchItem(const cEvent *Event, cChannel *Channel);
};

cMenuWhatsOnSearchItem::cMenuWhatsOnSearchItem(const cEvent *Event, cChannel *Channel)
{
  event = Event;
  channel = Channel;
  char *buffer = NULL;
  int TimerMatch;
  char t = Timers.GetMatch(Event, &TimerMatch) ? (TimerMatch == tmFull) ? 'T' : 't' : ' ';
  char v = event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ';
  char r = event->IsRunning() ? '*' : ' ';
  asprintf(&buffer, "%d\t%.*s\t%s\t%c%c%c\t%s", channel->Number(), 6, channel->Name(), event->GetTimeString(), t, v, r, event->Title());
  SetText(buffer, false);
}

// --- cMenuWhatsOnSearchEPG -------------------------------------------------------



cMenuWhatsOnSearchEPGItem::cMenuWhatsOnSearchEPGItem(const cEvent *EventInfo)
{
  eventInfo = EventInfo;
  char *buffer = NULL;
  cChannel *channel = Channels.GetByChannelID(eventInfo->ChannelID(), true);
  if(eventInfo->ShortText())
     asprintf(&buffer, "%.*s\t%.*s\t%.*s\t%s~%s", 10, channel ? channel->Name() : "???",
                        6, eventInfo->GetDateString(), 5, eventInfo->GetTimeString(), eventInfo->Title(), eventInfo->ShortText());
  else
     asprintf(&buffer, "%.*s\t%.*s\t%.*s\t%s", 10, channel ? channel->Name() : "???",
                        6, eventInfo->GetDateString(), 5, eventInfo->GetTimeString(), eventInfo->Title());
  SetText(buffer, false);
}

// --- cMenuWhatsOnSearchEPG -------------------------------------------------------

const cEvent *cMenuWhatsOnSearchEPG::scheduleEventInfo = NULL;
int cMenuWhatsOnSearchEPG::currentChannel = 0;

cMenuWhatsOnSearchEPG::cMenuWhatsOnSearchEPG(const char *p2)
:cOsdMenu(tr("additional starting times"), 13, 6, 6)
{
  asprintf(&m_pSearchString, "%s", p2);
  searchExt = NULL;
  m_bSort = true;

  BuildList();
}

cMenuWhatsOnSearchEPG::cMenuWhatsOnSearchEPG(cSearchExt* SearchExt)
:cOsdMenu(tr("Search results"), 12, 6, 6)
{
  searchExt = SearchExt;
  m_pSearchString = NULL;
  m_bSort = true;

  BuildList();
}

cEvent * cMenuWhatsOnSearchEPG::GetEventByTitle(const cSchedule *schedules, const char *p2, const cEvent *Start) 
{
  if(!p2) return NULL;

  cEvent *pe = NULL;
  cEvent *p1=schedules->Events()->First();
// first we have to search for the Start item
  if(Start) {
     cEvent *p;
     for (p = schedules->Events()->First(); p && p!=Start; p = schedules->Events()->Next(p));
     if(p)
        p1=schedules->Events()->Next(p);
     }

  time_t tNow=time(NULL);
  for (cEvent *p = p1; p; p = schedules->Events()->Next(p)) {
     if(!p) {
        break;
     }
     if (tNow < p->EndTime()) {
        if(p->Title() && !strcasecmp(p2, p->Title())) {
           pe=p;
           break;
        }
     }
  }
  return pe;
}

cEvent * cMenuWhatsOnSearchEPG::GetEventBySearchExt(const cSchedule *schedules, cSearchExt* SearchExt, const cEvent *Start) 
{
  if(!SearchExt) return NULL;

  cEvent *pe = NULL;
  cEvent *p1=schedules->Events()->First();
  // first we have to search for the Start item
  if(Start) {
     cEvent *p;
     for (p = schedules->Events()->First(); p && p!=Start; p = schedules->Events()->Next(p));
     if(p)
        p1=schedules->Events()->Next(p);
     }

  time_t tNow=time(NULL);
  char* szTest = NULL;
  char* searchText = NULL;
  asprintf(&searchText, "%s", SearchExt->search);

  int searchStart = 0, searchStop = 0;
  if (SearchExt->useTime)
  {
	searchStart = SearchExt->startTime;
	searchStop = SearchExt->stopTime;
	if (searchStop < searchStart)
		searchStop += 2400;
  }

  for (cEvent *p = p1; p; p = schedules->Events()->Next(p))
  {
     if(!p) 
     {
        break;
     }
     if (szTest)
     {
	free(szTest);
	szTest = NULL;
     }
     asprintf(&szTest, "%s%s%s", (SearchExt->useTitle?p->Title():""),
				 (SearchExt->useSubtitle?p->ShortText():""),
				 (SearchExt->useDescription?p->Description():""));     
	
     if (tNow < p->EndTime()) 
     {
        if (!SearchExt->useCase)
        {
		ToLower(szTest);
                ToLower(searchText);
        }
	
       	bool bTesting=false;
        if (SearchExt->mode == 0 && !strstr(szTest, searchText)) 
		continue;
        else
		bTesting = true;
        if (SearchExt->mode > 0)
        { 
		char *pstrSearchToken;
        	bool bFirst=true;
	        char *pstrSearch=strdup(searchText);
       		pstrSearchToken=strtok(pstrSearch, " ,;|");
        	while(pstrSearchToken) {
           		if(szTest && strstr(szTest, pstrSearchToken)) { 
              			if(SearchExt->mode==1) { // means AND
                 			if(bFirst) {
                    				bTesting=true;
                    				bFirst=false;
                 			} else
                    			bTesting&=true;
              			}
              			else
                 			bTesting|=true;
           		} else {// not found!!
              			if(SearchExt->mode==1) { // means AND
                 			bTesting=false;
                 			bFirst=false;
              			}
           		}
          		pstrSearchToken=strtok(NULL, " ,;|");
		}
		free(pstrSearch);

        }
        if(!bTesting)
		continue;
       

        if (SearchExt->useTime)
        {
		time_t tEvent = p->StartTime();
		struct tm tmEvent = *localtime(&tEvent);
		int eventStart = tmEvent.tm_hour*100 + tmEvent.tm_min;	
		int eventStart2 = tmEvent.tm_hour*100 + tmEvent.tm_min + 2400;	
		if ((eventStart < searchStart || eventStart > searchStop) &&
		    (eventStart2 < searchStart || eventStart2 > searchStop))
			continue;
        }
	pe=p;
	break;
     }
  }
  if (szTest)
	free(szTest);
  free(searchText);
  return pe;
}

bool cMenuWhatsOnSearchEPG::BuildList(void)
{
  cSchedulesLock schedulesLock;
  const cSchedules *schedules;
  schedules = cSchedules::Schedules(schedulesLock);
  if(!schedules) {
     esyslog("cMenuWhatsOnSearchEPG::BuildList scheduler is empty!");
     return false;
  }
  const cSchedule *Schedule = schedules->First();
  const cEvent **pArray = NULL;
  const cEvent *pPrevEvent = NULL;
  int num = 0;

  Clear();
  while (Schedule) {
        if (searchExt && searchExt->useChannel && searchExt->channel)
	{
		cChannel* channel = Channels.GetByChannelID(Schedule->ChannelID(),true);
		if (channel && searchExt->channel->Number() != channel->Number())
		{
			Schedule = (const cSchedule *)schedules->Next(Schedule);
			continue;
		}
	}

        pArray = (const cEvent **)realloc(pArray, (num + 1) * sizeof(cEvent *));
        pPrevEvent = NULL;
        do {
	   if (!searchExt)
           	pArray[num] = GetEventByTitle(Schedule, m_pSearchString, pPrevEvent);
	   else
           	pArray[num] = GetEventBySearchExt(Schedule, searchExt, pPrevEvent);
           pPrevEvent = pArray[num];
           if (pArray[num]) {
              cChannel *channel = Channels.GetByChannelID(pArray[num]->ChannelID(), true);
              if (channel) {
                 num++;
                 pArray = (const cEvent **)realloc(pArray, (num + 1) * sizeof(cEvent *));
              }
           }
        } while(pPrevEvent);
        Schedule = (const cSchedule *)schedules->Next(Schedule);
        }

  qsort(pArray, num, sizeof(cEvent *), m_bSort? CompareEventTime: CompareEventChannel);

  for (int a = 0; a < num; a++)
     Add(new cMenuWhatsOnSearchEPGItem(pArray[a]));

  delete pArray;
  
  SetHelp(tr("Record"), m_bSort? tr("by Channel"):tr("by Time"), NULL, NULL);
  Display();
  return true;
}


eOSState cMenuWhatsOnSearchEPG::Record(void)
{
  cMenuWhatsOnSearchEPGItem *item = (cMenuWhatsOnSearchEPGItem *)Get(Current());
  if (item) {
     cTimer *timer = new cTimer(item->eventInfo);
     cTimer *t = Timers.GetTimer(timer);
     if (t) {
        delete timer;
        timer = t;
        }
     return AddSubMenu(new cMenuEditTimer(timer, !t));
     }
  return osContinue;
}


eOSState cMenuWhatsOnSearchEPG::Switch(void)
{
  cMenuWhatsOnSearchItem *item = (cMenuWhatsOnSearchItem *)Get(Current());
  if (item) {
     cChannel *channel = Channels.GetByChannelID(item->event->ChannelID(), true);
     if (channel && cDevice::PrimaryDevice()->SwitchChannel(channel, true))
        return osEnd;
     }
  Skins.Message(mtInfo, tr("Can't switch channel!")); 
  return osContinue;
}

eOSState cMenuWhatsOnSearchEPG::ProcessKey(eKeys Key)
{
  eOSState state = osContinue;

     switch (Key) {
       case kRecord:
       case kRed:    return Record();
       case k0:      state=osBack; break;
       case kGreen:  {
                        m_bSort=!m_bSort;
			BuildList();
                     }
                     break;
       case kYellow: break; //return Switch(); 
       case kBlue:   break; //return Search();
       case kOk:     if(HasSubMenu()) {
                        state = cOsdMenu::ProcessKey(Key);
                        break;
                     }
		     if (Count())
                        return AddSubMenu(new cMenuEventSearch(((cMenuWhatsOnSearchEPGItem *)Get(Current()))->eventInfo, 0));
                     break;
       default:      
                     state = cOsdMenu::ProcessKey(Key);
     }
  return state;
}




// --- cMenuWhatsOnSearch ----------------------------------------------------------

class cMenuWhatsOnSearch : public cOsdMenu {
private:
  eOSState Record(void);
  eOSState ExtendedSearch(void);
  static int currentChannel;
  static const cEvent *scheduleEvent;
public:
  cMenuWhatsOnSearch(const cSchedules *Schedules, bool Now, int CurrentChannelNr);
  static int CurrentChannel(void) { return currentChannel; }
  static void SetCurrentChannel(int ChannelNr) { currentChannel = ChannelNr; }
  static const cEvent *ScheduleEvent(void);
  virtual eOSState ProcessKey(eKeys Key);
  };

int cMenuWhatsOnSearch::currentChannel = 0;
const cEvent *cMenuWhatsOnSearch::scheduleEvent = NULL;

cMenuWhatsOnSearch::cMenuWhatsOnSearch(const cSchedules *Schedules, bool Now, int CurrentChannelNr)
:cOsdMenu(Now ? tr("What's on now?") : tr("What's on next?"), CHNUMWIDTH, 7, 6, 4)
{
  for (cChannel *Channel = Channels.First(); Channel; Channel = Channels.Next(Channel)) {
      if (!Channel->GroupSep()) {
         const cSchedule *Schedule = Schedules->GetSchedule(Channel->GetChannelID());
         if (Schedule) {
            const cEvent *Event = Now ? Schedule->GetPresentEvent() : Schedule->GetFollowingEvent();
            if (Event)
               Add(new cMenuWhatsOnSearchItem(Event, Channel), Channel->Number() == CurrentChannelNr);
            }
         }
      }
  currentChannel = CurrentChannelNr;
  SetHelp(Count() ? tr("Repeatings") : NULL, Now ? tr("Next") : tr("Now"), tr("Button$Schedule"), tr("Search"));
}

const cEvent *cMenuWhatsOnSearch::ScheduleEvent(void)
{
  const cEvent *ei = scheduleEvent;
  scheduleEvent = NULL;
  return ei;
}


eOSState cMenuWhatsOnSearch::Record(void)
{
  cMenuWhatsOnSearchItem *item = (cMenuWhatsOnSearchItem *)Get(Current());
  if (item) {
     cTimer *timer = new cTimer(item->event);
     cTimer *t = Timers.GetTimer(timer);
     if (t) {
        delete timer;
        timer = t;
        }
     return AddSubMenu(new cMenuEditTimer(timer, !t));
     }
  return osContinue;
}

eOSState cMenuWhatsOnSearch::ExtendedSearch(void)
{
    return AddSubMenu(new cMenuEPGSearchExt());
}

eOSState cMenuWhatsOnSearch::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  if (state == osUnknown) {
     switch (Key) {
       case kRecord:
       case kRed:    
       case k0: 
		if(HasSubMenu()) {
                        state = Record();
                        break;
                     } 
	     if (Count()) {
                       cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
                       if (mi) {
                          if (mi->event) {
                             return AddSubMenu(new cMenuWhatsOnSearchEPG(mi->event->Title()));
                             }
                          }
                     }
       case kYellow: state = osBack;
                     // continue with kGreen
       case kGreen:  {
                       cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
                       if (mi) {
                          scheduleEvent = mi->event;
                          currentChannel = mi->channel->Number();
                          }
                     }
                     break;
       case kBlue:   return ExtendedSearch();
		     break;
       case kOk:     if (Count())
                        return AddSubMenu(new cMenuEventSearch(((cMenuWhatsOnSearchItem *)Get(Current()))->event, true));
                     break;
       default:      break;
       }
     }
  return state;
}

// --- cMenuScheduleItem -----------------------------------------------------

class cMenuScheduleItem : public cOsdItem {
public:
  const cEvent *event;
  cMenuScheduleItem(const cEvent *Event);
};

cMenuScheduleItem::cMenuScheduleItem(const cEvent *Event)
{
  event = Event;
  char *buffer = NULL;
  int TimerMatch;
  char t = Timers.GetMatch(Event, &TimerMatch) ? (TimerMatch == tmFull) ? 'T' : 't' : ' ';
  char v = event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ';
  char r = event->IsRunning() ? '*' : ' ';
  asprintf(&buffer, "%.*s\t%s\t%c%c%c\t%s", 6, event->GetDateString(), event->GetTimeString(), t, v, r, event->Title());
  SetText(buffer, false);
}

// --- cMenuSchedule ---------------------------------------------------------

class cMenuScheduleSearch : public cOsdMenu {
private:
  cSchedulesLock schedulesLock;
  const cSchedules *schedules;
  bool now, next;
  int otherChannel;
  eOSState Record(void);
  eOSState ExtendedSearch(void);
  void PrepareSchedule(cChannel *Channel);
public:
  cMenuScheduleSearch(void);
  virtual ~cMenuScheduleSearch();
  virtual eOSState ProcessKey(eKeys Key);
  };

cMenuScheduleSearch::cMenuScheduleSearch(void)
:cOsdMenu("", 7, 6, 4)
{
  now = next = false;
  otherChannel = 0;
  cChannel *channel = Channels.GetByNumber(cDevice::CurrentChannel());
  if (channel) {
     cMenuWhatsOnSearch::SetCurrentChannel(channel->Number());
     schedules = cSchedules::Schedules(schedulesLock);
     PrepareSchedule(channel);
     SetHelp(Count() ? tr("Repeatings") : NULL, tr("Now"), tr("Next"), tr("Search"));
     }
}

cMenuScheduleSearch::~cMenuScheduleSearch()
{
  cMenuWhatsOnSearch::ScheduleEvent(); // makes sure any posted data is cleared
}

void cMenuScheduleSearch::PrepareSchedule(cChannel *Channel)
{
  Clear();
  char *buffer = NULL;
  asprintf(&buffer, tr("Schedule - %s"), Channel->Name());
  SetTitle(buffer);
  free(buffer);
  if (schedules) {
     const cSchedule *Schedule = schedules->GetSchedule(Channel->GetChannelID());
     if (Schedule) {
        const cEvent *PresentEvent = Schedule->GetPresentEvent(Channel->Number() == cDevice::CurrentChannel());
        time_t now = time(NULL) - Setup.EPGLinger * 60;
        for (const cEvent *Event = Schedule->Events()->First(); Event; Event = Schedule->Events()->Next(Event)) {
            if (Event->EndTime() > now || Event == PresentEvent)
               Add(new cMenuScheduleItem(Event), Event == PresentEvent);
            }
        }
     }
}

eOSState cMenuScheduleSearch::Record(void)
{
  cMenuScheduleItem *item = (cMenuScheduleItem *)Get(Current());
  if (item) {
     cTimer *timer = new cTimer(item->event);
     cTimer *t = Timers.GetTimer(timer);
     if (t) {
        delete timer;
        timer = t;
        }
     return AddSubMenu(new cMenuEditTimer(timer, !t));
     }
  return osContinue;
}

eOSState cMenuScheduleSearch::ExtendedSearch(void)
{
    return AddSubMenu(new cMenuEPGSearchExt());
}

eOSState cMenuScheduleSearch::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  if (state == osUnknown) {
     switch (Key) {
       case kRecord:
       case kRed:
       case k0:
		if(HasSubMenu()) {
                        state = Record();
                        break;
                     } 
      		if (Count()) {
                       cMenuWhatsOnSearchItem *mi = (cMenuWhatsOnSearchItem *)Get(Current());
                       if (mi) {
                          if (mi->event) {
                             return AddSubMenu(new cMenuWhatsOnSearchEPG(mi->event->Title()));
                             }
                          }
                     }

       case kGreen:  if (schedules) {
                        if (!now && !next) {
                           int ChannelNr = 0;
                           if (Count()) {
                              cChannel *channel = Channels.GetByChannelID(((cMenuScheduleItem *)Get(Current()))->event->ChannelID(), true);
                              if (channel)
                                 ChannelNr = channel->Number();
                              }
                           now = true;
                           return AddSubMenu(new cMenuWhatsOnSearch(schedules, true, ChannelNr));
                           }
                        now = !now;
                        next = !next;
                        return AddSubMenu(new cMenuWhatsOnSearch(schedules, now, cMenuWhatsOnSearch::CurrentChannel()));
                        }
       case kYellow: if (schedules)
                        return AddSubMenu(new cMenuWhatsOnSearch(schedules, false, cMenuWhatsOnSearch::CurrentChannel()));
                     break;
       case kBlue:   return ExtendedSearch();
                     break;
       case kOk:     if (Count())
                        return AddSubMenu(new cMenuEventSearch(((cMenuScheduleItem *)Get(Current()))->event, otherChannel));
                     break;
       default:      break;
       }
     }
  else if (!HasSubMenu()) {
     now = next = false;
     const cEvent *ei = cMenuWhatsOnSearch::ScheduleEvent();
     if (ei) {
        cChannel *channel = Channels.GetByChannelID(ei->ChannelID(), true);
        if (channel) {
           PrepareSchedule(channel);
           if (channel->Number() != cDevice::CurrentChannel()) {
              otherChannel = channel->Number();
              SetHelp(Count() ? tr("Search") : NULL, tr("Now"), tr("Next"), tr("Switch"));
              }
           Display();
           }
        }
     }
  return state;
}


class cPluginEpgsearch : public cPlugin {
private:
  // Add any member variables or functions you may need here.
public:
  cPluginEpgsearch(void);
  virtual ~cPluginEpgsearch();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return tr(DESCRIPTION); }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Housekeeping(void);
  virtual const char *MainMenuEntry(void) {  return (EPGSearchConfig.hidemenu ? NULL : tr(MAINMENUENTRY)); }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  };

cPluginEpgsearch::cPluginEpgsearch(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
}

cPluginEpgsearch::~cPluginEpgsearch()
{
  // Clean up after yourself!
}

const char *cPluginEpgsearch::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return NULL;
}

bool cPluginEpgsearch::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  return true;
}

bool cPluginEpgsearch::Initialize(void)
{
  // Initialize any background activities the plugin shall perform.
  RegisterI18n(Phrases);
  return true;
}

bool cPluginEpgsearch::Start(void)
{
  // Start any background activities the plugin shall perform.
  return true;
}

void cPluginEpgsearch::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.
}

cOsdObject *cPluginEpgsearch::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.
  return new cMenuScheduleSearch();
}

cMenuSetupPage *cPluginEpgsearch::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return new cMenuEPGSearchSetup;
}

bool cPluginEpgsearch::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  if      (!strcasecmp(Name, "HideMenu"))       EPGSearchConfig.hidemenu       = atoi(Value);
  return true;
}

VDRPLUGINCREATOR(cPluginEpgsearch); // Don't touch this!
